/*
 * ip6.c
 *
 */

#include "config.h"

#include <sys/types.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "dnet.h"
#include "aton.h"
#include "mod.h"

void
ip6_usage(void)
{
	fprintf(stderr, "Usage: dnet ip6 [tc|flow|next|hlim|src|dst "
	    "<value>] ... \n");
	exit(1);
}

int
ip6_main(int argc, char *argv[])
{
	uint8_t tc = 0, next, hlim = IP6_HLIM_MAX;
	uint32_t flow = 0;
	struct addr addr;
	struct ip6_addr src, dst;
	u_char *p, buf[IP6_LEN_MAX];	/* XXX */
	char *name, *value;
	int c, len;
	
	srand(time(NULL));

	next = rand() & 0xff;
	memset(buf, 0, sizeof buf);

	for (c = 1; c + 1 < argc; c += 2) {
		name = argv[c];
		value = argv[c + 1];
		
		if (strcmp(name, "tc") == 0)
			tc = atoi(value);
		else if (strcmp(name, "flow") == 0)
			flow = atoi(value);
		else if (strcmp(name, "hlim") == 0)
			hlim = atoi(value);
		else if (strcmp(name, "next") == 0) {
			if (proto_aton(value, &next) < 0)
				ip6_usage();
		} else if (strcmp(name, "src") == 0) {
			if (addr_aton(value, &addr) < 0 || addr.addr_type != ADDR_TYPE_IP6)
				ip6_usage();
			memcpy(&src, &addr.addr_ip6, IP6_ADDR_LEN);
		} else if (strcmp(name, "dst") == 0) {
			if (addr_aton(value, &addr) < 0 || addr.addr_type != ADDR_TYPE_IP6)
				ip6_usage();
			memcpy(&dst, &addr.addr_ip6, IP6_ADDR_LEN);
		} else
			ip6_usage();
	}
	argc -= c;
	argv += c;
	
	if (argc != 0)
		ip6_usage();
	
	if (isatty(STDIN_FILENO))
		errx(1, "can't read IPv6 payload from tty");
	
	p = buf + IP6_HDR_LEN;
	len = sizeof(buf) - (p - buf);
	
	while ((c = read(STDIN_FILENO, p, len)) > 0) {
		p += c;
		len -= c;
	}
	len = p - buf;

	ip6_pack_hdr(buf, tc, flow, len - IP6_HDR_LEN, next, hlim, src, dst);

	ip6_checksum(buf, len);
	
	if (write(STDOUT_FILENO, buf, len) != len)
		err(1, "write");

	return (0);
}

struct mod mod_ip6 = {
	"ip6",
	MOD_TYPE_ENCAP,
	ip6_main
};
